var x_PI = 3.14159265358979324 * 3000.0 / 180.0
var PI = 3.1415926535897932384626
var a = 6378245.0
var ee = 0.00669342162296594323

function bd09togcj02(lng, lat) {
    var bd_lon = Number.parseFloat(lng)
    var bd_lat = Number.parseFloat(lat)
    var x_pi = 3.14159265358979324 * 3000.0 / 180.0
    var x = bd_lon - 0.0065
    var y = bd_lat - 0.006
    var z = Math.sqrt(x * x + y * y) - 0.00002 * Math.sin(y * x_pi)
    var theta = Math.atan2(y, x) - 0.000003 * Math.cos(x * x_pi)
    var gg_lng = z * Math.cos(theta)
    var gg_lat = z * Math.sin(theta)
    return {
        lng: gg_lng,
        lat: gg_lat
    }
}

function bd09towgs84(bd_lon, bd_lat) {
    var res = bd09togcj02(bd_lon, bd_lat)
    return gcj02towgs84(res.lng, res.lat)
}

function wgs84tobd09(lng, lat){
    var res = wgs84togcj02(lng, lat)
    return gcj02tobd09(res.lng, res.lat)
}

function gcj02tobd09(lng, lat) {
    lng = Number.parseFloat(lng)
    lat = Number.parseFloat(lat)
    var z = Math.sqrt(lng * lng + lat * lat) + 0.00002 * Math.sin(lat * x_PI)
    var theta = Math.atan2(lat, lng) + 0.000003 * Math.cos(lng * x_PI)
    var bd_lng = z * Math.cos(theta) + 0.0065
    var bd_lat = z * Math.sin(theta) + 0.006
    return { lng: Number.parseFloat(bd_lng), lat: Number.parseFloat(bd_lat) }
}

function wgs84togcj02(lng, lat) {
    lng = Number.parseFloat(lng)
    lat = Number.parseFloat(lat)
    var dlat = transformlat(lng - 105.0, lat - 35.0)
    var dlng = transformlng(lng - 105.0, lat - 35.0)
    var radlat = lat / 180.0 * PI
    var magic = Math.sin(radlat)
    magic = 1 - ee * magic * magic
    var sqrtmagic = Math.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * PI)
    dlng = (dlng * 180.0) / (a / sqrtmagic * Math.cos(radlat) * PI)
    var mglat = lat + dlat
    var mglng = lng + dlng
    return {
        lng: mglng,
        lat: mglat,
    }
}

function gcj02towgs84(lng, lat) {
    lng = Number.parseFloat(lng)
    lat = Number.parseFloat(lat)
    var dlat = transformlat(lng - 105.0, lat - 35.0)
    var dlng = transformlng(lng - 105.0, lat - 35.0)
    var radlat = lat / 180.0 * PI
    var magic = Math.sin(radlat)
    magic = 1 - ee * magic * magic
    var sqrtmagic = Math.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * PI)
    dlng = (dlng * 180.0) / (a / sqrtmagic * Math.cos(radlat) * PI)
    var mglat = lat + dlat
    var mglng = lng + dlng
    return {
        lng: lng * 2 - mglng,
        lat: lat * 2 - mglat,
    }
}

function transformlat(lng, lat) {
    lng = Number.parseFloat(lng)
    lat = Number.parseFloat(lat)
    var ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + 0.1 * lng * lat + 0.2 * Math.sqrt(Math.abs(lng))
    ret += (20.0 * Math.sin(6.0 * lng * PI) + 20.0 * Math.sin(2.0 * lng * PI)) * 2.0 / 3.0
    ret += (20.0 * Math.sin(lat * PI) + 40.0 * Math.sin(lat / 3.0 * PI)) * 2.0 / 3.0
    ret += (160.0 * Math.sin(lat / 12.0 * PI) + 320 * Math.sin(lat * PI / 30.0)) * 2.0 / 3.0
    return ret
}

function transformlng(lng, lat) {
    lng = Number.parseFloat(lng)
    lat = Number.parseFloat(lat)
    var ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + 0.1 * lng * lat + 0.1 * Math.sqrt(Math.abs(lng))
    ret += (20.0 * Math.sin(6.0 * lng * PI) + 20.0 * Math.sin(2.0 * lng * PI)) * 2.0 / 3.0
    ret += (20.0 * Math.sin(lng * PI) + 40.0 * Math.sin(lng / 3.0 * PI)) * 2.0 / 3.0
    ret += (150.0 * Math.sin(lng / 12.0 * PI) + 300.0 * Math.sin(lng / 30.0 * PI)) * 2.0 / 3.0
    return ret
}

function transformLocal(local, current, target) {

    var res = JSON.parse(JSON.stringify(local))

    var isSame = false
    var currentPro
    var targetPro

    if(current){
        currentPro = current.options.projection
        targetPro = target.options.projection
        if(current.options.projection != target.options.projection){
            isSame = false
        }else{
            isSame = true
        }
    }

    if(!isSame){

        if(currentPro == 'EPSG3857'){
            if(targetPro == 'EPSG4326'){
                var _t = gcj02towgs84(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG4326'
            }else if(targetPro == 'baidu'){
                var _t = gcj02tobd09(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'baidu'
            }
        }else if(currentPro == 'EPSG4326'){
            if(targetPro == 'EPSG3857'){
                var _t = wgs84togcj02(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG3857'
            }else if(targetPro == 'baidu'){
                var _t = wgs84tobd09(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'baidu'
            }
        }else if(currentPro == 'baidu'){
            if(targetPro == 'EPSG3857'){
                var _t = bd09togcj02(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG3857'
            }else if(targetPro == 'EPSG4326'){
                var _t = bd09towgs84(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG4326'
            }
        }else if(currentPro == 'EPSG4490'){
            if(targetPro == 'EPSG3857'){
                var _t = bd09togcj02(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG3857'
            }else if(targetPro == 'EPSG4326'){
                var _t = bd09towgs84(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG4326'
            }else if(targetPro == 'baidu'){
                var _t = wgs84tobd09(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'baidu'
            }else{
                console.log('坐标转换失败，暂未支持该坐标的转换')
            }
        }else if(currentPro == 'EPSG4160'){
            if(targetPro == 'EPSG3857'){
                var _t = bd09togcj02(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG3857'
            }else if(targetPro == 'EPSG4326'){
                var _t = bd09towgs84(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG4326'
            }else if(targetPro == 'baidu'){
                var _t = wgs84tobd09(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'baidu'
            }else{
                console.log('坐标转换失败，暂未支持该坐标的转换')
            }
        }else if(currentPro == 'EPSG4214'){
            if(targetPro == 'EPSG3857'){
                var _t = bd09togcj02(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG3857'
            }else if(targetPro == 'EPSG4326'){
                var _t = bd09towgs84(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'EPSG4326'
            }else if(targetPro == 'baidu'){
                var _t = wgs84tobd09(res.lng, res.lat)
                res.lng = Number.parseFloat(_t.lng)
                res.lat = Number.parseFloat(_t.lat)
                res.projection = 'baidu'
            }else{
                console.log('坐标转换失败，暂未支持该坐标的转换')
            }
        }

    }

    return res
}